=== modified file '.bzrignore'
--- .bzrignore	2014-02-28 09:25:47 +0000
+++ .bzrignore	2014-03-05 16:43:49 +0000
@@ -1,3 +1,5 @@
+# For all dependencies run from source
+./branches/*
 ./docs/_build
 .deps
 *.egg-info

=== added directory 'branches'
=== added file 'branches/README'
--- branches/README	1970-01-01 00:00:00 +0000
+++ branches/README	2014-03-05 16:43:49 +0000
@@ -0,0 +1,6 @@
+This directory contain branches used by the ubuntu-ci-services project.
+
+It is not under version control in the project but can hold any branch.
+
+It is used to cache local copies of some dependencies that needs to be run
+from source.

=== modified file 'juju-deployer/deploy.py'
--- juju-deployer/deploy.py	2014-03-04 22:45:12 +0000
+++ juju-deployer/deploy.py	2014-03-05 16:43:49 +0000
@@ -14,6 +14,15 @@
 from distutils import spawn
 from itertools import chain
 
+import bzrlib
+from bzrlib import (
+    builtins,
+    plugin,
+)
+
+
+HERE = os.path.abspath(os.path.dirname(__file__))
+
 # the juju-deployer/ directory.
 deployer_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
 # ci-utils probably isn't in our pythonpath
@@ -21,6 +30,10 @@
 from ci_utils import dump_stack
 
 
+bzrlib.initialize()
+plugin.load_plugins()
+
+
 def check_environment():
     '''Cheetah requires environment variables for populating our templates'''
     cheetah_vars = {
@@ -108,52 +121,71 @@
         yield ('-c', os.path.join(d, f))
 
 
-def append_python_path(path):
-    path = os.path.abspath(path)
-    if os.environ.get('PYTHONPATH'):
-        os.environ['PYTHONPATH'] += ':%s' % path
-    else:
-        os.environ['PYTHONPATH'] = '%s' % path
-
-
-def install(branch, revno=0):
-    """Put a copy of a bzr branch in a temporary directory."""
-
-    temp_dir = tempfile.mkdtemp()
-    try:
-        cmd = ['bzr', 'co', '--lightweight']
-        if revno > 0:
-            cmd.append('-r%d' % revno)
-        cmd += [branch, temp_dir]
-        subprocess.check_call(cmd)
-        append_python_path(temp_dir)
-    except:
-        shutil.rmtree(temp_dir)
-        raise
-
-    return temp_dir
-
-
-def install_jujuclient():
+def prepend_path(env_var, path):
+    """Put our dependencies paths first so they don't get hidden."""
+    val = os.environ.get(env_var, None)
+    paths = (val and val.split(os.path.pathsep)) or []
+    paths.insert(0, path)
+    os.environ[env_var] = os.path.pathsep.join(paths)
+
+
+def get_or_pull_branch(src, dst, revision=None, out=None, err=None):
+    """Get a branch at a given revision.
+
+    If the branch already exists, pulls the given revision.
+
+    :param src: The url to get the branch from (any url recognized by bzr).
+
+    :param dst: The directory where the branch will reside.
+
+    :param revision: The optional revision to set the branch to.
+    """
+    with bzrlib.initialize(stdout=out, stderr=err):
+        if os.path.exists(dst):
+            cmd = builtins.cmd_pull()
+            args = [src, '-d', dst, '--overwrite']
+        else:
+            cmd = builtins.cmd_branch()
+            args = [src, dst]
+        if revision is not None:
+            args.extend(['--revision', revision])
+        cmd.run_argv_aliases(args)
+
+
+def install_jujuclient(where=None):
     """Install python-jujuclient from bzr. Revision r23 works around problems
     we were seeing with env watches (LP: #1284183)."""
-    return install('lp:python-jujuclient', revno=24)
-
-
-def install_deployer():
-    """Install juju-deployer from bzr. We need juju-deployer from tip to have
-    JUJU_REPOSITORY work properly (r87)."""
-
-    deployer = install('lp:juju-deployer', revno=103)
-    os.environ['PATH'] = '%s:%s' % (deployer, os.environ['PATH'])
-    deployer_bin = os.path.join(deployer, 'juju-deployer')
+    if where is None:
+        where = os.path.join(HERE, '..', 'branches')
+    dest = os.path.abspath(os.path.join(where, 'charmworldlib'))
+    get_or_pull_branch('lp:python-jujuclient', dest, 'revno:24')
+    prepend_path('PYTHONPATH', dest)
+    return dest
+
+
+def install_deployer(where=None):
+    """Install juju-deployer from bzr."""
+    if where is None:
+        where = os.path.join(HERE, '..', 'branches')
+    dest = os.path.abspath(os.path.join(where, 'juju-deployer'))
+    # For an obscure reason tag:0.3.1 is not in lp:juju-deployer
+    get_or_pull_branch('lp:juju-deployer', dest, revision='revno:103')
+    prepend_path('PYTHONPATH', dest)
+    # FIXME: This duplicates setup.py and shouldn't be needed. This specific
+    # piece (juju-deployer) is a weird beast, it installs from source (not pip
+    # nor package), it requires amulet which itself requires urllib3 (among
+    # others), it's used both in unit tests that are executed in a venv and in
+    # ./tests/run.py which is not executed in venv. My head hurts -- vila
+    # 2014-02-12
+    deployer_bin = os.path.join(dest, 'juju-deployer')
     with open(deployer_bin, 'w') as fp:
-        tmpl = '#!/bin/sh\nPYTHONPATH=%s\nexec python %s/deployer/cli.py $*'
-        fp.write(tmpl % (os.environ['PYTHONPATH'], deployer))
+        tmpl = '#!/bin/sh\nPYTHONPATH={}\nexec python {}/deployer/cli.py $*'
+        fp.write(tmpl.format(os.environ['PYTHONPATH'], dest))
     st = os.stat(deployer_bin)
     mode = st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
     os.chmod(deployer_bin, mode)
-    return deployer
+    prepend_path('PATH', dest)
+    return dest
 
 
 def remove_branch(branch_dir):
@@ -196,7 +228,7 @@
             local_charms[release].append(charm)
             charmdir = os.path.join(charmsdir, release, charm)
             if os.path.exists(os.path.join(charmdir, 'Makefile')):
-                print('Building charm: %s' % charm)
+                print('Building charm: {}'.format(charm))
                 subprocess.check_call(['make'], cwd=charmdir)
     return clean_func
 
@@ -252,16 +284,15 @@
     check_bootstrapped()
     check_dependencies()
 
-    temp_dir = install_jujuclient()
-    atexit.register(lambda: remove_branch(temp_dir))
-
-    temp_dir = install_deployer()
-    atexit.register(lambda: remove_branch(temp_dir))
+    install_jujuclient()
+    install_deployer()
 
     # A place to put our generated .yaml files.
     temp_dir = tempfile.mkdtemp()
-    atexit.register(lambda: shutil.rmtree(temp_dir))
+    atexit.register(shutil.rmtree, temp_dir)
     working_dir = os.path.join(temp_dir, 'deployer')
+    # Copy all existing .yaml files, they don't need to be populated but
+    # juju-deployer expect them in 'working_dir'.
     shutil.copytree(deployer_dir, working_dir)
 
     if not args.branch_payload and not args.reuse_payload:

=== modified file 'juju-deployer/test_deploy.py'
--- juju-deployer/test_deploy.py	2014-02-22 16:30:18 +0000
+++ juju-deployer/test_deploy.py	2014-03-05 16:43:49 +0000
@@ -8,6 +8,10 @@
 import shutil
 import subprocess
 
+
+from ucitests import fixtures
+
+
 sys.path.append(os.path.abspath(os.path.dirname(sys.argv[0])))
 import deploy
 
@@ -57,11 +61,11 @@
             deploy.check_bootstrapped()
 
     def test_install_deployer(self):
+        fixtures.set_uniq_cwd(self)
         with mock.patch.dict('os.environ', {'PYTHONPATH': ''}):
-            deployer_dir = deploy.install_deployer()
-            self.addCleanup(lambda: shutil.rmtree(deployer_dir))
+            deployer_dir = deploy.install_deployer('.')
             self.assertTrue(os.path.exists(os.path.join(deployer_dir, '.bzr')))
-            self.assertEqual(os.environ['PYTHONPATH'], '%s' % deployer_dir)
+            self.assertEqual(deployer_dir, os.environ['PYTHONPATH'])
             deployer_script = os.path.join(deployer_dir, 'juju-deployer')
             self.assertTrue(os.access(deployer_script, os.X_OK))
             self.assertTrue(os.path.exists(deployer_script))

=== modified file 'tests/run.py'
--- tests/run.py	2014-02-28 09:25:47 +0000
+++ tests/run.py	2014-03-05 16:43:49 +0000
@@ -25,6 +25,10 @@
 logging.basicConfig(level=logging.INFO, format='%(msg)s')
 log = logging.getLogger('integration-test-runner')
 
+
+HERE = os.path.abspath(os.path.dirname(__file__))
+
+
 # deploy.py probably isn't in our pythonpath
 sys.path.append(os.path.join(os.path.dirname(__file__), '../juju-deployer'))
 import deploy
@@ -162,13 +166,23 @@
     subprocess.check_call(cmd)
 
 
-def install_charmworld():
+def install_charmworld(where=None):
     """Recent versions of amulet need charmworldlib. Install it."""
-    return deploy.install('lp:charmworldlib')
-
-
-def install_amulet():
-    return deploy.install('lp:amulet')
+    if where is None:
+        where = os.path.join(HERE, '..', 'branches')
+    dest = os.path.abspath(os.path.join(where, 'charmworldlib'))
+    deploy.get_or_pull_branch('lp:charmworldlib', dest)
+    deploy.prepend_path('PYTHONPATH', dest)
+    return dest
+
+
+def install_amulet(where=None):
+    if where is None:
+        where = os.path.join(HERE, '..', 'branches')
+    dest = os.path.abspath(os.path.join(where, 'amulet'))
+    deploy.get_or_pull_branch('lp:amulet', dest)
+    deploy.prepend_path('PYTHONPATH', dest)
+    return dest
 
 
 def remove_branch(branch_dir):

=== modified file 'tests/test_run.py'
--- tests/test_run.py	2014-02-17 10:13:29 +0000
+++ tests/test_run.py	2014-03-05 16:43:49 +0000
@@ -13,6 +13,9 @@
 from novaclient.exceptions import NotFound
 
 
+from ucitests import fixtures
+
+
 logging.disable(logging.CRITICAL)
 
 
@@ -63,11 +66,12 @@
         self.assertEqual(sorted(run.get_tests(temp_dir)), sorted(expect))
 
     def test_install_amulet(self):
+        fixtures.set_uniq_cwd(self)
         with mock.patch.dict('os.environ', {'PYTHONPATH': ''}):
-            amulet_dir = run.install_amulet()
-            self.addCleanup(lambda: shutil.rmtree(amulet_dir))
+            amulet_dir = run.install_amulet('.')
+            self.addCleanup(shutil.rmtree, amulet_dir)
             self.assertTrue(os.path.exists(os.path.join(amulet_dir, '.bzr')))
-            self.assertEqual(os.environ['PYTHONPATH'], '%s' % amulet_dir)
+            self.assertEqual(amulet_dir, os.environ['PYTHONPATH'])
 
     @mock.patch('run._get_control_bucket',
                 return_value='juju-d911b43e427c204cfcfd8c7276c4b2aa')

